@@ -11,11 +11,6 @@ def is_submodule(path):
 
 __all__ = [p.stem for p in Path(__file__).parent.iterdir() if is_submodule(p)]
 
-import sys, logging
-
-logging.basicConfig(stream=sys.stdout, level=logging.INFO)
-logger = logging.getLogger(__name__)
-
 from . import maps
 from . import helpers
 from .data import *
