@@ -1,12 +1,12 @@
 from __future__ import annotations
 import itertools
-import logging
 import math
 import random
 import time
 import warnings
 from collections import Counter
 from typing import Any, Dict, List, Optional, Set, Tuple, Union, TYPE_CHECKING
+from contextlib import suppress
 from s2clientprotocol import sc2api_pb2 as sc_pb
 
 from .cache import property_cache_forever, property_cache_once_per_frame, property_cache_once_per_frame_no_copy
@@ -44,7 +44,7 @@ from .units import Units
 from .game_data import Cost
 from .unit_command import UnitCommand
 
-logger = logging.getLogger(__name__)
+from loguru import logger
 
 if TYPE_CHECKING:
     from .game_info import GameInfo, Ramp
@@ -76,32 +76,36 @@ class BotAI(DistanceCalculation):
         self.all_units: Units = Units([], self)
         self.units: Units = Units([], self)
         self.workers: Units = Units([], self)
-        self.townhalls: Units = Units([], self)
+        self.larva: Units = Units([], self)
         self.structures: Units = Units([], self)
+        self.townhalls: Units = Units([], self)
         self.gas_buildings: Units = Units([], self)
+        self.all_own_units: Units = Units([], self)
         self.enemy_units: Units = Units([], self)
         self.enemy_structures: Units = Units([], self)
+        self.all_enemy_units: Units = Units([], self)
         self.resources: Units = Units([], self)
         self.destructables: Units = Units([], self)
         self.watchtowers: Units = Units([], self)
         self.mineral_field: Units = Units([], self)
         self.vespene_geyser: Units = Units([], self)
         self.placeholders: Units = Units([], self)
-        self.larva: Units = Units([], self)
         self.techlab_tags: Set[int] = set()
         self.reactor_tags: Set[int] = set()
-        self.minerals: int = None
-        self.vespene: int = None
-        self.supply_army: float = None
-        self.supply_workers: float = None  # Doesn't include workers in production
-        self.supply_cap: float = None
-        self.supply_used: float = None
-        self.supply_left: float = None
-        self.idle_worker_count: int = None
-        self.army_count: int = None
-        self.warp_gate_count: int = None
+        self.minerals: int = 50
+        self.vespene: int = 0
+        self.supply_army: float = 0
+        self.supply_workers: float = 12  # Doesn't include workers in production
+        self.supply_cap: float = 15
+        self.supply_used: float = 12
+        self.supply_left: float = 3
+        self.idle_worker_count: int = 0
+        self.army_count: int = 0
+        self.warp_gate_count: int = 0
         self.actions: List[UnitCommand] = []
         self.blips: Set[Blip] = set()
+        self.race: Race = None
+        self.enemy_race: Race = None
         self._units_created: Counter = Counter()
         self._unit_tags_seen_this_game: Set[int] = set()
         self._units_previous_map: Dict[int, Unit] = dict()
@@ -180,7 +184,7 @@ class BotAI(DistanceCalculation):
         Check if alert is triggered in the current step.
         Possible alerts are listed here https://github.com/Blizzard/s2client-proto/blob/e38efed74c03bec90f74b330ea1adda9215e655f/s2clientprotocol/sc2api.proto#L679-L702
 
-        Example use:
+        Example use::
 
             from sc2.data import Alert
             if self.alert(Alert.AddOnComplete):
@@ -870,6 +874,7 @@ class BotAI(DistanceCalculation):
         max_distance: int = 20,
         random_alternative: bool = True,
         placement_step: int = 2,
+        addon_place: bool = False,
     ) -> Optional[Point2]:
         """ Finds a placement location for building.
 
@@ -893,7 +898,9 @@ class BotAI(DistanceCalculation):
         else:  # AbilityId
             building = self._game_data.abilities[building.value]
 
-        if await self.can_place(building, near):
+        if await self.can_place(building, near) and (
+            not addon_place or await self.can_place(UnitTypeId.SUPPLYDEPOT, near.offset((2.5, -0.5)))
+        ):
             return near
 
         if max_distance == 0:
@@ -911,6 +918,14 @@ class BotAI(DistanceCalculation):
             ]
             res = await self._client.query_building_placement(building, possible_positions)
             possible = [p for r, p in zip(res, possible_positions) if r == ActionResult.Success]
+
+            if addon_place:
+                res = await self._client.query_building_placement(
+                    self._game_data.units[UnitTypeId.SUPPLYDEPOT.value].creation_ability,
+                    [p.offset((2.5, -0.5)) for p in possible],
+                )
+                possible = [p for r, p in zip(res, possible) if r == ActionResult.Success]
+
             if not possible:
                 continue
 
@@ -1503,21 +1518,17 @@ class BotAI(DistanceCalculation):
             # action: UnitCommand
             # current_action: UnitOrder
             current_action = action.unit.orders[0]
-            if current_action.ability.id != action.ability:
+            if current_action.ability.id != action.ability and current_action.ability.exact_id != action.ability:
                 # Different action, return True
                 return True
-            try:
+            with suppress(AttributeError):
                 if current_action.target == action.target.tag:
                     # Same action, remove action if same target unit
                     return False
-            except AttributeError:
-                pass
-            try:
+            with suppress(AttributeError):
                 if action.target.x == current_action.target.x and action.target.y == current_action.target.y:
                     # Same action, remove action if same target position
                     return False
-            except AttributeError:
-                pass
             return True
         return True
 
@@ -1578,7 +1589,7 @@ class BotAI(DistanceCalculation):
 
         :param pos: """
         assert isinstance(pos, (Point2, Unit)), f"pos is not of type Point2 or Unit"
-        pos = pos.position.to2.rounded
+        pos = pos.position.rounded
         return self._game_info.pathing_grid[pos] == 1
 
     def is_visible(self, pos: Union[Point2, Unit]) -> bool:
@@ -1587,7 +1598,7 @@ class BotAI(DistanceCalculation):
         :param pos: """
         # more info: https://github.com/Blizzard/s2client-proto/blob/9906df71d6909511907d8419b33acc1a3bd51ec0/s2clientprotocol/spatial.proto#L19
         assert isinstance(pos, (Point2, Unit)), f"pos is not of type Point2 or Unit"
-        pos = pos.position.to2.rounded
+        pos = pos.position.rounded
         return self.state.visibility[pos] == 2
 
     def has_creep(self, pos: Union[Point2, Unit]) -> bool:
@@ -1595,7 +1606,7 @@ class BotAI(DistanceCalculation):
 
         :param pos: """
         assert isinstance(pos, (Point2, Unit)), f"pos is not of type Point2 or Unit"
-        pos = pos.position.to2.rounded
+        pos = pos.position.rounded
         return self.state.creep[pos] == 1
 
     def _prepare_start(self, client, player_id, game_info, game_data, realtime: bool = False):
@@ -1637,7 +1648,7 @@ class BotAI(DistanceCalculation):
         """
         # Set attributes from new state before on_step."""
         self.state: GameState = state  # See game_state.py
-        # update pathing grid
+        # update pathing grid, which unfortunately is in GameInfo instead of GameState
         self._game_info.pathing_grid: PixelMap = PixelMap(
             proto_game_info.game_info.start_raw.pathing_grid, in_bits=True, mirrored=False
         )
@@ -1668,24 +1679,29 @@ class BotAI(DistanceCalculation):
         self.army_count: int = state.common.army_count
         self._time_before_step: float = time.perf_counter()
 
+        if self.enemy_race == Race.Random and self.all_enemy_units:
+            self.enemy_race = Race(self.all_enemy_units.first.race)
+
     def _prepare_units(self):
         # Set of enemy units detected by own sensor tower, as blips have less unit information than normal visible units
         self.blips: Set[Blip] = set()
-        self.placeholders: Units = Units([], self)
+        self.all_units: Units = Units([], self)
         self.units: Units = Units([], self)
+        self.workers: Units = Units([], self)
+        self.larva: Units = Units([], self)
         self.structures: Units = Units([], self)
+        self.townhalls: Units = Units([], self)
+        self.gas_buildings: Units = Units([], self)
+        self.all_own_units: Units = Units([], self)
         self.enemy_units: Units = Units([], self)
         self.enemy_structures: Units = Units([], self)
-        self.mineral_field: Units = Units([], self)
-        self.vespene_geyser: Units = Units([], self)
+        self.all_enemy_units: Units = Units([], self)
         self.resources: Units = Units([], self)
         self.destructables: Units = Units([], self)
         self.watchtowers: Units = Units([], self)
-        self.all_units: Units = Units([], self)
-        self.workers: Units = Units([], self)
-        self.townhalls: Units = Units([], self)
-        self.gas_buildings: Units = Units([], self)
-        self.larva: Units = Units([], self)
+        self.mineral_field: Units = Units([], self)
+        self.vespene_geyser: Units = Units([], self)
+        self.placeholders: Units = Units([], self)
         self.techlab_tags: Set[int] = set()
         self.reactor_tags: Set[int] = set()
 
@@ -1726,6 +1742,7 @@ class BotAI(DistanceCalculation):
                         self.destructables.append(unit_obj)
                 # Alliance.Self.value = 1
                 elif alliance == 1:
+                    self.all_own_units.append(unit_obj)
                     unit_id = unit_obj.type_id
                     if unit_obj.is_structure:
                         self.structures.append(unit_obj)
@@ -1756,6 +1773,7 @@ class BotAI(DistanceCalculation):
                             self.larva.append(unit_obj)
                 # Alliance.Enemy.value = 4
                 elif alliance == 4:
+                    self.all_enemy_units.append(unit_obj)
                     if unit_obj.is_structure:
                         self.enemy_structures.append(unit_obj)
                     else:
@@ -1885,7 +1903,7 @@ class BotAI(DistanceCalculation):
                     await self.on_enemy_unit_left_vision(enemy_unit_tag)
         if self.enemy_structures:
             visible_enemy_structures = self.enemy_structures.tags
-            for enemy_structure_tag in self._enemy_units_previous_map.keys():
+            for enemy_structure_tag in self._enemy_structures_previous_map.keys():
                 if enemy_structure_tag not in visible_enemy_structures:
                     await self.on_enemy_unit_left_vision(enemy_structure_tag)
 
