@@ -1,8 +1,6 @@
 from __future__ import annotations
-import logging
 from typing import Any, Dict, Iterable, List, Optional, Set, Tuple, Union, TYPE_CHECKING
 
-from s2clientprotocol import common_pb2 as common_pb
 from s2clientprotocol import debug_pb2 as debug_pb
 from s2clientprotocol import query_pb2 as query_pb
 from s2clientprotocol import raw_pb2 as raw_pb
@@ -20,7 +18,7 @@ from .renderer import Renderer
 from .unit import Unit
 from .units import Units
 
-logger = logging.getLogger(__name__)
+from loguru import logger
 
 
 class Client(Protocol):
@@ -82,7 +80,6 @@ class Client(Protocol):
             req = sc_pb.RequestJoinGame(race=race.value, options=ifopts)
 
         if portconfig:
-            req.shared_port = portconfig.shared
             req.server_ports.game_port = portconfig.server[0]
             req.server_ports.base_port = portconfig.server[1]
 
@@ -210,24 +207,10 @@ class Client(Protocol):
         assert isinstance(start, (Point2, Unit))
         assert isinstance(end, Point2)
         if isinstance(start, Point2):
-            result = await self._execute(
-                query=query_pb.RequestQuery(
-                    pathing=[
-                        query_pb.RequestQueryPathing(
-                            start_pos=common_pb.Point2D(x=start.x, y=start.y),
-                            end_pos=common_pb.Point2D(x=end.x, y=end.y),
-                        )
-                    ]
-                )
-            )
+            path = [query_pb.RequestQueryPathing(start_pos=start.as_Point2D, end_pos=end.as_Point2D)]
         else:
-            result = await self._execute(
-                query=query_pb.RequestQuery(
-                    pathing=[
-                        query_pb.RequestQueryPathing(unit_tag=start.tag, end_pos=common_pb.Point2D(x=end.x, y=end.y))
-                    ]
-                )
-            )
+            path = [query_pb.RequestQueryPathing(unit_tag=start.tag, end_pos=end.as_Point2D)]
+        result = await self._execute(query=query_pb.RequestQuery(pathing=path))
         distance = float(result.query.pathing[0].distance)
         if distance <= 0.0:
             return None
@@ -247,25 +230,12 @@ class Client(Protocol):
         assert isinstance(zipped_list[0][0], (Point2, Unit)), f"{type(zipped_list[0][0])}"
         assert isinstance(zipped_list[0][1], Point2), f"{type(zipped_list[0][1])}"
         if isinstance(zipped_list[0][0], Point2):
-            results = await self._execute(
-                query=query_pb.RequestQuery(
-                    pathing=(
-                        query_pb.RequestQueryPathing(
-                            start_pos=common_pb.Point2D(x=p1.x, y=p1.y), end_pos=common_pb.Point2D(x=p2.x, y=p2.y)
-                        )
-                        for p1, p2 in zipped_list
-                    )
-                )
+            path = (
+                query_pb.RequestQueryPathing(start_pos=p1.as_Point2D, end_pos=p2.as_Point2D) for p1, p2 in zipped_list
             )
         else:
-            results = await self._execute(
-                query=query_pb.RequestQuery(
-                    pathing=(
-                        query_pb.RequestQueryPathing(unit_tag=p1.tag, end_pos=common_pb.Point2D(x=p2.x, y=p2.y))
-                        for p1, p2 in zipped_list
-                    )
-                )
-            )
+            path = (query_pb.RequestQueryPathing(unit_tag=p1.tag, end_pos=p2.as_Point2D) for p1, p2 in zipped_list)
+        results = await self._execute(query=query_pb.RequestQuery(pathing=path))
         return [float(d.distance) for d in results.query.pathing]
 
     async def _query_building_placement_fast(
@@ -280,9 +250,7 @@ class Client(Protocol):
         result = await self._execute(
             query=query_pb.RequestQuery(
                 placements=(
-                    query_pb.RequestQueryBuildingPlacement(
-                        ability_id=ability.id.value, target_pos=common_pb.Point2D(x=position[0], y=position[1])
-                    )
+                    query_pb.RequestQueryBuildingPlacement(ability_id=ability.id.value, target_pos=position.as_Point2D)
                     for position in positions
                 ),
                 ignore_resource_requirements=ignore_resources,
@@ -303,9 +271,7 @@ class Client(Protocol):
         result = await self._execute(
             query=query_pb.RequestQuery(
                 placements=(
-                    query_pb.RequestQueryBuildingPlacement(
-                        ability_id=ability.id.value, target_pos=common_pb.Point2D(x=position[0], y=position[1])
-                    )
+                    query_pb.RequestQueryBuildingPlacement(ability_id=ability.id.value, target_pos=position.as_Point2D)
                     for position in positions
                 ),
                 ignore_resource_requirements=ignore_resources,
@@ -336,6 +302,19 @@ class Client(Protocol):
             return [[AbilityId(a.ability_id) for a in b.abilities] for b in result.query.abilities][0]
         return [[AbilityId(a.ability_id) for a in b.abilities] for b in result.query.abilities]
 
+    async def query_available_abilities_with_tag(
+        self, units: Union[List[Unit], Units], ignore_resource_requirements: bool = False
+    ) -> Dict[int, Set[AbilityId]]:
+        """ Query abilities of multiple units """
+
+        result = await self._execute(
+            query=query_pb.RequestQuery(
+                abilities=(query_pb.RequestQueryAvailableAbilities(unit_tag=unit.tag) for unit in units),
+                ignore_resource_requirements=ignore_resource_requirements,
+            )
+        )
+        return {b.unit_tag: {AbilityId(a.ability_id) for a in b.abilities} for b in result.query.abilities}
+
     async def chat_send(self, message: str, team_only: bool):
         """ Writes a message to the chat """
         ch = ChatChannel.Team if team_only else ChatChannel.Broadcast
@@ -390,7 +369,7 @@ class Client(Protocol):
                         create_unit=debug_pb.DebugCreateUnit(
                             unit_type=unit_type.value,
                             owner=owner_id,
-                            pos=common_pb.Point2D(x=position.x, y=position.y),
+                            pos=position.as_Point2D,
                             quantity=amount_of_units,
                         )
                     )
@@ -425,9 +404,7 @@ class Client(Protocol):
                 actions=[
                     sc_pb.Action(
                         action_raw=raw_pb.ActionRaw(
-                            camera_move=raw_pb.ActionRawCameraMove(
-                                center_world_space=common_pb.Point(x=position.x, y=position.y)
-                            )
+                            camera_move=raw_pb.ActionRawCameraMove(center_world_space=position.to3.as_Point)
                         )
                     )
                 ]
@@ -444,11 +421,7 @@ class Client(Protocol):
         await self._execute(
             obs_action=sc_pb.RequestObserverAction(
                 actions=[
-                    sc_pb.ObserverAction(
-                        camera_move=sc_pb.ActionObserverCameraMove(
-                            world_pos=common_pb.Point2D(x=position.x, y=position.y)
-                        )
-                    )
+                    sc_pb.ObserverAction(camera_move=sc_pb.ActionObserverCameraMove(world_pos=position.as_Point2D))
                 ]
             )
         )
@@ -462,9 +435,7 @@ class Client(Protocol):
         assert isinstance(position, (Point2, Point3))
         action = sc_pb.Action(
             action_render=spatial_pb.ActionSpatial(
-                camera_move=spatial_pb.ActionSpatialCameraMove(
-                    center_minimap=common_pb.PointI(x=position.x, y=position.y)
-                )
+                camera_move=spatial_pb.ActionSpatialCameraMove(center_minimap=position.as_PointI)
             )
         )
         await self._execute(action=sc_pb.RequestAction(actions=[action]))
@@ -703,13 +674,8 @@ class Client(Protocol):
 
 
 class DrawItem:
-    def to_debug_point(self, point: Union[Unit, Point2, Point3]) -> common_pb.Point:
-        """ Helper function for point conversion """
-        if isinstance(point, Unit):
-            point = point.position3d
-        return common_pb.Point(x=point.x, y=point.y, z=getattr(point, "z", 0))
-
-    def to_debug_color(self, color: Union[tuple, Point3]):
+    @staticmethod
+    def to_debug_color(color: Union[tuple, Point3]):
         """ Helper function for color conversion """
         if color is None:
             return debug_pb.Color(r=255, g=255, b=255)
@@ -740,7 +706,7 @@ class DrawItemScreenText(DrawItem):
         return debug_pb.DebugText(
             color=self.to_debug_color(self._color),
             text=self._text,
-            virtual_pos=self.to_debug_point(self._start_point),
+            virtual_pos=self._start_point.to3.as_Point,
             world_pos=None,
             size=self._font_size,
         )
@@ -761,7 +727,7 @@ class DrawItemWorldText(DrawItem):
             color=self.to_debug_color(self._color),
             text=self._text,
             virtual_pos=None,
-            world_pos=self.to_debug_point(self._start_point),
+            world_pos=self._start_point.as_Point,
             size=self._font_size,
         )
 
@@ -777,7 +743,7 @@ class DrawItemLine(DrawItem):
 
     def to_proto(self):
         return debug_pb.DebugLine(
-            line=debug_pb.Line(p0=self.to_debug_point(self._start_point), p1=self.to_debug_point(self._end_point)),
+            line=debug_pb.Line(p0=self._start_point.as_Point, p1=self._end_point.as_Point),
             color=self.to_debug_color(self._color),
         )
 
@@ -793,9 +759,7 @@ class DrawItemBox(DrawItem):
 
     def to_proto(self):
         return debug_pb.DebugBox(
-            min=self.to_debug_point(self._start_point),
-            max=self.to_debug_point(self._end_point),
-            color=self.to_debug_color(self._color),
+            min=self._start_point.as_Point, max=self._end_point.as_Point, color=self.to_debug_color(self._color),
         )
 
     def __hash__(self):
@@ -810,7 +774,7 @@ class DrawItemSphere(DrawItem):
 
     def to_proto(self):
         return debug_pb.DebugSphere(
-            p=self.to_debug_point(self._start_point), r=self._radius, color=self.to_debug_color(self._color)
+            p=self._start_point.as_Point, r=self._radius, color=self.to_debug_color(self._color)
         )
 
     def __hash__(self):
