@@ -1,22 +1,90 @@
 import asyncio
-import logging
+from aiohttp import ClientWebSocketResponse, ClientSession
 import time
 import six
 import json
-import os
 import mpyq
+import os
+import sys
+import platform
+import portpicker
+import signal
 import async_timeout
+from dataclasses import dataclass
+from typing import Dict, List, Union
+
 from s2clientprotocol import sc2api_pb2 as sc_pb
 
 from .client import Client
-from .data import CreateGameError, Result
+from .controller import Controller
+from .data import CreateGameError, Result, Status
 from .game_state import GameState
-from .player import Bot, Human
+from .maps import Map
+from .player import AbstractPlayer, Bot, BotProcess, Human
 from .portconfig import Portconfig
 from .protocol import ConnectionAlreadyClosed, ProtocolError
-from .sc2process import SC2Process
+from .proxy import Proxy
+from .sc2process import SC2Process, kill_switch
 
-logger = logging.getLogger(__name__)
+from loguru import logger
+
+# Set the global logging level
+logger.remove()
+logger.add(sys.stdout, level="INFO")
+
+
+@dataclass
+class GameMatch:
+    """Dataclass for hosting a match of SC2.
+    This contains all of the needed information for RequestCreateGame.
+    :param sc2_config: dicts of arguments to unpack into sc2process's construction, one per player
+        second sc2_config will be ignored if only one sc2_instance is spawned
+        e.g. sc2_args=[{"fullscreen": True}, {}]: only player 1's sc2instance will be fullscreen
+    :param game_time_limit: The time (in seconds) until a match is artificially declared a Tie
+    """
+
+    map_sc2: Map
+    players: List[AbstractPlayer]
+    realtime: bool = False
+    random_seed: int = None
+    disable_fog: bool = None
+    sc2_config: List[Dict] = None
+    game_time_limit: int = None
+
+    def __post_init__(self):
+        # avoid players sharing names
+        if len(self.players) > 1 and self.players[0].name is not None and self.players[0].name == self.players[1].name:
+            self.players[1].name += "2"
+
+        if self.sc2_config is not None:
+            if isinstance(self.sc2_config, dict):
+                self.sc2_config = [self.sc2_config]
+            if len(self.sc2_config) == 0:
+                self.sc2_config = [{}]
+            while len(self.sc2_config) < len(self.players):
+                self.sc2_config += self.sc2_config
+            self.sc2_config = self.sc2_config[: len(self.players)]
+
+    @property
+    def needed_sc2_count(self) -> int:
+        return sum(player.needs_sc2 for player in self.players)
+
+    @property
+    def host_game_kwargs(self) -> Dict:
+        return {
+            "map_settings": self.map_sc2,
+            "players": self.players,
+            "realtime": self.realtime,
+            "random_seed": self.random_seed,
+            "disable_fog": self.disable_fog,
+        }
+
+    def __repr__(self):
+        p1 = self.players[0]
+        p1 = p1.name if p1.name else p1
+        p2 = self.players[1]
+        p2 = p2.name if p2.name else p2
+        return f"Map: {self.map_sc2.name}, {p1} vs {p2}, realtime:{self.realtime}, seed={self.random_seed}"
 
 
 class SlidingTimeWindow:
@@ -108,10 +176,10 @@ async def _play_game_ai(client, player_id, ai, realtime, step_time_limit, game_t
         return client._game_result[player_id]
     gs = GameState(state.observation)
     proto_game_info = await client._execute(game_info=sc_pb.RequestGameInfo())
-    ai._prepare_step(gs, proto_game_info)
-    await ai.on_before_start()
-    ai._prepare_first_step()
     try:
+        ai._prepare_step(gs, proto_game_info)
+        await ai.on_before_start()
+        ai._prepare_first_step()
         await ai.on_start()
     except Exception as e:
         logger.exception(f"AI on_start threw an error")
@@ -245,21 +313,24 @@ async def _play_game_ai(client, player_id, ai, realtime, step_time_limit, game_t
 
 
 async def _play_game(
-    player, client, realtime, portconfig, step_time_limit=None, game_time_limit=None, rgb_render_config=None
-):
+    player, client: Client, realtime, portconfig, step_time_limit=None, game_time_limit=None, rgb_render_config=None
+) -> Result:
     assert isinstance(realtime, bool), repr(realtime)
 
     player_id = await client.join_game(
         player.name, player.race, portconfig=portconfig, rgb_render_config=rgb_render_config
     )
-    logging.info(f"Player {player_id} - {player.name if player.name else str(player)}")
+    logger.info(f"Player {player_id} - {player.name if player.name else str(player)}")
 
     if isinstance(player, Human):
         result = await _play_game_human(client, player_id, realtime, game_time_limit)
     else:
         result = await _play_game_ai(client, player_id, player.ai, realtime, step_time_limit, game_time_limit)
 
-    logging.info(f"Result for player {player_id} - {player.name if player.name else str(player)}: {result._name_}")
+    logger.info(
+        f"Result for player {player_id} - {player.name if player.name else str(player)}: "
+        f"{result._name_ if isinstance(result, Result) else result}"
+    )
 
     return result
 
@@ -316,17 +387,10 @@ async def _play_replay(client, ai, realtime=False, player_id=0):
         logger.debug(f"Running AI step, it={iteration} {gs.game_loop * 0.725 * (1 / 16):.2f}s")
 
         try:
-            if realtime:
-                # Issue event like unit created or unit destroyed
-                await ai.issue_events()
-                await ai.on_step(iteration)
-                await ai._after_step()
-            else:
-
-                # Issue event like unit created or unit destroyed
-                await ai.issue_events()
-                await ai.on_step(iteration)
-                await ai._after_step()
+            # Issue event like unit created or unit destroyed
+            await ai.issue_events()
+            await ai.on_step(iteration)
+            await ai._after_step()
 
         except Exception as e:
             if isinstance(e, ProtocolError) and e.is_game_over_error:
@@ -362,7 +426,7 @@ async def _play_replay(client, ai, realtime=False, player_id=0):
         iteration += 1
 
 
-async def _setup_host_game(server, map_settings, players, realtime, random_seed=None, disable_fog=None):
+async def _setup_host_game(server: Controller, map_settings, players, realtime, random_seed=None, disable_fog=None):
     r = await server.create_game(map_settings, players, realtime, random_seed, disable_fog)
     if r.create_game.HasField("error"):
         err = f"Could not create game: {CreateGameError(r.create_game.error)}"
@@ -377,7 +441,7 @@ async def _setup_host_game(server, map_settings, players, realtime, random_seed=
 async def _host_game(
     map_settings,
     players,
-    realtime,
+    realtime=False,
     portconfig=None,
     save_replay_as=None,
     step_time_limit=None,
@@ -411,7 +475,7 @@ async def _host_game(
             await client.leave()
             await client.quit()
         except ConnectionAlreadyClosed:
-            logging.error(f"Connection was closed before the game ended")
+            logger.error(f"Connection was closed before the game ended")
             return None
 
         return result
@@ -439,7 +503,7 @@ async def _host_game_aiter(
                     await client.save_replay(save_replay_as)
                 await client.leave()
             except ConnectionAlreadyClosed:
-                logging.error(f"Connection was closed before the game ended")
+                logger.error(f"Connection was closed before the game ended")
                 return
 
             new_players = yield result
@@ -472,7 +536,7 @@ async def _join_game(
             await client.leave()
             await client.quit()
         except ConnectionAlreadyClosed:
-            logging.error(f"Connection was closed before the game ended")
+            logger.error(f"Connection was closed before the game ended")
             return None
 
         return result
@@ -531,3 +595,250 @@ def run_replay(ai, replay_path, realtime=False, observed_id=0):
         _host_replay(replay_path, ai, realtime, portconfig, base_build, data_version, observed_id)
     )
     return result
+
+
+async def play_from_websocket(
+    ws_connection: Union[str, ClientWebSocketResponse],
+    player: AbstractPlayer,
+    realtime: bool = False,
+    portconfig: Portconfig = None,
+    save_replay_as=None,
+    game_time_limit: int = None,
+    should_close=True,
+):
+    """Use this to play when the match is handled externally e.g. for bot ladder games.
+    Portconfig MUST be specified if not playing vs Computer.
+    :param ws_connection: either a string("ws://{address}:{port}/sc2api") or a ClientWebSocketResponse object
+    :param should_close: closes the connection if True. Use False if something else will reuse the connection
+
+    e.g. ladder usage: play_from_websocket("ws://127.0.0.1:5162/sc2api", MyBot, False, portconfig=my_PC)
+    """
+    session = None
+    try:
+        if isinstance(ws_connection, str):
+            session = ClientSession()
+            ws_connection = await session.ws_connect(ws_connection, timeout=120)
+            should_close = True
+        client = Client(ws_connection)
+        result = await _play_game(player, client, realtime, portconfig, game_time_limit=game_time_limit)
+        if save_replay_as is not None:
+            await client.save_replay(save_replay_as)
+    except ConnectionAlreadyClosed:
+        logger.error(f"Connection was closed before the game ended")
+        return None
+    finally:
+        if should_close:
+            await ws_connection.close()
+            if session:
+                await session.close()
+
+    return result
+
+
+async def run_match(controllers: List[Controller], match: GameMatch, close_ws=True):
+    await _setup_host_game(controllers[0], **match.host_game_kwargs)
+
+    # Setup portconfig beforehand, so all players use the same ports
+    startport = None
+    portconfig = None
+    if match.needed_sc2_count > 1:
+        if any(isinstance(player, BotProcess) for player in match.players):
+            portconfig = Portconfig.contiguous_ports()
+            # Most ladder bots generate their server and client ports as [s+2, s+3], [s+4, s+5]
+            startport = portconfig.server[0] - 2
+        else:
+            portconfig = Portconfig()
+
+    proxies = []
+    coros = []
+    i = 0
+    for player in match.players:
+        if player.needs_sc2:
+            if isinstance(player, BotProcess):
+                pport = portpicker.pick_unused_port()
+                p = Proxy(controllers[i], player, pport, match.game_time_limit, match.realtime)
+                proxies.append(p)
+                coros.append(p.play_with_proxy(startport))
+            else:
+                coros.append(
+                    play_from_websocket(
+                        controllers[i]._ws,
+                        player,
+                        match.realtime,
+                        portconfig,
+                        should_close=close_ws,
+                        game_time_limit=match.game_time_limit,
+                    )
+                )
+            i += 1
+
+    # async_results = await asyncio.wait_for(asyncio.gather(*coros, return_exceptions=True), timeout=None)
+    async_results = await asyncio.gather(*coros, return_exceptions=True)
+
+    if not isinstance(async_results, list):
+        async_results = [async_results]
+    for i, a in enumerate(async_results):
+        if isinstance(a, Exception):
+            logger.error(f"Exception[{a}] thrown by {[p for p in match.players if p.needs_sc2][i]}")
+
+    return process_results(match.players, async_results)
+
+
+def process_results(players, async_results):
+    opp_res = {Result.Victory: Result.Defeat, Result.Defeat: Result.Victory, Result.Tie: Result.Tie}
+    result: Dict[AbstractPlayer, Result] = {}
+    i = 0
+    for player in players:
+        if player.needs_sc2:
+            if sum(r == Result.Victory for r in async_results) <= 1:
+                result[player] = async_results[i]
+            else:
+                result[player] = Result.Undecided
+            i += 1
+        else:  # computer
+            other_result = async_results[0]
+            result[player] = opp_res[other_result] if other_result in opp_res else None
+
+    return result
+
+
+async def maintain_SCII_count(count: int, controllers: List[Controller], proc_args: List[Dict] = None):
+    """Modifies the given list of controllers to reflect the desired amount of SCII processes"""
+    # kill unhealthy ones.
+    if controllers:
+        toRemove = []
+        alive = await asyncio.wait_for(
+            asyncio.gather(*(c.ping() for c in controllers if not c._ws.closed), return_exceptions=True), timeout=20
+        )
+        i = 0  # for alive
+        for controller in controllers:
+            if controller._ws.closed:
+                if not controller._process._session.closed:
+                    await controller._process._session.close()
+                toRemove.append(controller)
+            else:
+                if not isinstance(alive[i], sc_pb.Response):
+                    try:
+                        await controller._process._close_connection()
+                    finally:
+                        toRemove.append(controller)
+                i += 1
+        for c in toRemove:
+            c._process._clean()
+            if c._process in kill_switch._to_kill:
+                kill_switch._to_kill.remove(c._process)
+            controllers.remove(c)
+
+    # spawn more
+    if len(controllers) < count:
+        needed = count - len(controllers)
+        if proc_args:
+            index = len(controllers) % len(proc_args)
+        else:
+            proc_args = [{} for _ in range(needed)]
+            index = 0
+        extra = [SC2Process(**proc_args[(index + _) % len(proc_args)]) for _ in range(needed)]
+        logger.info(f"Creating {needed} more SC2 Processes")
+        for k in range(3):  # try thrice
+            if platform.system() == "Linux":
+                # Works on linux: start one client after the other
+                new_controllers = [await asyncio.wait_for(sc.__aenter__(), timeout=50) for sc in extra]
+            else:
+                # Doesnt seem to work on linux: starting 2 clients nearly at the same time
+                new_controllers = await asyncio.wait_for(
+                    asyncio.gather(*[sc.__aenter__() for sc in extra], return_exceptions=True), timeout=50
+                )
+
+            controllers.extend(c for c in new_controllers if isinstance(c, Controller))
+            if len(controllers) == count:
+                await asyncio.wait_for(asyncio.gather(*(c.ping() for c in controllers)), timeout=20)
+                break
+            extra = [
+                extra[i] for i, result in enumerate(new_controllers) if not isinstance(new_controllers, Controller)
+            ]
+        else:
+            logger.critical("Could not launch sufficient SC2")
+            raise RuntimeError
+
+    # kill excess
+    while len(controllers) > count:
+        proc = controllers.pop()
+        proc = proc._process
+        logger.info(f"Removing SCII listening to {proc._port}")
+        await proc._close_connection()
+        proc._clean()
+        if proc in kill_switch._to_kill:
+            kill_switch._to_kill.remove(proc)
+
+
+def run_multiple_games(matches: List[GameMatch]):
+    return asyncio.get_event_loop().run_until_complete(a_run_multiple_games(matches))
+
+
+async def a_run_multiple_games(matches: List[GameMatch]):
+    """Run multiple matches.
+    Non-python bots are supported.
+    When playing bot vs bot, this is less likely to fatally crash than repeating run_game()
+    """
+    if not matches:
+        return
+
+    results = []
+    controllers = []
+    for m in matches:
+        result = None
+        dont_restart = m.needed_sc2_count == 2
+        try:
+            await maintain_SCII_count(m.needed_sc2_count, controllers, m.sc2_config)
+            result = await run_match(controllers, m, close_ws=dont_restart)
+        except SystemExit as e:
+            logger.info(f"Game exit'ed as {e} during match {m}")
+        except Exception as e:
+            logger.info(f"Exception {e} thrown in match {m}")
+        finally:
+            if dont_restart:  # Keeping them alive after a non-computer match can cause crashes
+                await maintain_SCII_count(0, controllers, m.sc2_config)
+            results.append(result)
+    kill_switch.kill_all()
+    return results
+
+
+async def a_run_multiple_games_nokill(matches: List[GameMatch]):
+    """Run multiple matches while reusing SCII processes.
+    Prone to crashes and stalls
+    """
+    # FIXME: check whether crashes between bot-vs-bot are avoidable or not
+    if not matches:
+        return
+
+    # Start the matches
+    results = []
+    controllers = []
+    for m in matches:
+        logger.info(f"Starting match {1 + len(results)}: {m}")
+        result = None
+        try:
+            await maintain_SCII_count(m.needed_sc2_count, controllers, m.sc2_config)
+            result = await run_match(controllers, m, close_ws=False)
+        except SystemExit as e:
+            logger.critical(f"Game sys.exit'ed as {e} during match {m}")
+        except Exception as e:
+            logger.trace(f"Exception {e} thrown in match {m}")
+        finally:
+            for c in controllers:
+                try:
+                    await c.ping()
+                    if c._status != Status.launched:
+                        await c._execute(leave_game=sc_pb.RequestLeaveGame())
+                except Exception as e:
+                    if not (isinstance(e, ProtocolError) and e.is_game_over_error):
+                        logger.info(f"controller {c.__dict__} threw {e}")
+
+            results.append(result)
+
+    # Fire the killswitch manually, instead of letting the winning player fire it.
+    await asyncio.wait_for(asyncio.gather(*(c._process._close_connection() for c in controllers)), timeout=50)
+    kill_switch.kill_all()
+    signal.signal(signal.SIGINT, signal.SIG_DFL)
+
+    return results
