@@ -357,6 +357,17 @@ class Unit:
             speed *= SPEED_ALTERING_BUFFS.get(buff, 1)
         return speed
 
+    @property
+    def distance_per_step(self) -> float:
+        """ The distance a unit can move in one step. This does not take acceleration into account.
+        Useful for micro-retreat/pathfinding """
+        return (self.real_speed / 22.4) * self._bot_object.client.game_step
+
+    @property
+    def distance_to_weapon_ready(self) -> float:
+        """ Distance a unit can travel before it's weapon is ready to be fired again."""
+        return (self.real_speed / 22.4) * self.weapon_cooldown
+
     @property
     def is_mineral_field(self) -> bool:
         """ Checks if the unit is a mineral field. """
@@ -817,12 +828,15 @@ class Unit:
         return angle_difference < angle_error
 
     @property
-    def footprint_radius(self) -> float:
+    def footprint_radius(self) -> Optional[float]:
         """ For structures only.
         For townhalls this returns 2.5
         For barracks, spawning pool, gateway, this returns 1.5
         For supply depot, this returns 1
-        For sensor tower, creep tumor, this return 0.5 """
+        For sensor tower, creep tumor, this return 0.5
+
+        NOTE: This can be None if a building doesn't have a creation ability.
+        For rich vespene buildings, flying terran buildings, this returns None"""
         return self._bot_object._game_data.units[self._proto.unit_type].footprint_radius
 
     @property
@@ -1099,14 +1113,26 @@ class Unit:
 
     @property_immutable_cache
     def add_on_land_position(self) -> Point2:
-        """ If this unit is an addon (techlab, reactor), returns the position
-        where a terran building (BARRACKS, FACTORY, STARPORT) has to land to connect to this addon. """
+        """
+        If this unit is an addon (techlab, reactor), returns the position
+        where a terran building (BARRACKS, FACTORY, STARPORT) has to land to connect to this addon.
+
+        Why offset (-2.5, 0.5)? See description in 'add_on_position'
+        """
         return self.position.offset(Point2((-2.5, 0.5)))
 
     @property_immutable_cache
     def add_on_position(self) -> Point2:
-        """ If this unit is a terran production building (BARRACKS, FACTORY, STARPORT),
-        this property returns the position of where the addon should be, if it should build one or has one attached. """
+        """
+        If this unit is a terran production building (BARRACKS, FACTORY, STARPORT),
+        this property returns the position of where the addon should be, if it should build one or has one attached.
+
+        Why offset (2.5, -0.5)?
+        A barracks is of size 3x3. The distance from the center to the edge is 1.5.
+        An addon is 2x2 and the distance from the edge to center is 1.
+        The total distance from center to center on the x-axis is 2.5.
+        The distance from center to center on the y-axis is -0.5.
+        """
         return self.position.offset(Point2((2.5, -0.5)))
 
     @property_mutable_cache
@@ -1159,7 +1185,9 @@ class Unit:
     @property
     def surplus_harvesters(self) -> int:
         """ Returns a positive int if unit has too many harvesters mining,
-        a negative int if it has too few mining."""
+        a negative int if it has too few mining.
+        Will only works on townhalls, and gas buildings.
+        """
         return self._proto.assigned_harvesters - self._proto.ideal_harvesters
 
     @property_immutable_cache
@@ -1177,6 +1205,11 @@ class Unit:
             return self._proto.weapon_cooldown
         return -1
 
+    @property
+    def weapon_ready(self) -> bool:
+        """ Checks if the weapon is ready to be fired. """
+        return self.weapon_cooldown == 0
+
     @property
     def engaged_target_tag(self) -> int:
         # TODO What does this do?
@@ -1234,7 +1267,9 @@ class Unit:
             can_afford_check=can_afford_check,
         )
 
-    def build_gas(self, target_geysir: Unit, queue: bool = False, can_afford_check: bool = False) -> Union[UnitCommand, bool]:
+    def build_gas(
+        self, target_geysir: Unit, queue: bool = False, can_afford_check: bool = False
+    ) -> Union[UnitCommand, bool]:
         """ Orders unit to build another 'unit' at 'position'.
         Usage::
 
@@ -1256,7 +1291,9 @@ class Unit:
             can_afford_check=can_afford_check,
         )
 
-    def research(self, upgrade: UpgradeId, queue: bool = False, can_afford_check: bool = False) -> Union[UnitCommand, bool]:
+    def research(
+        self, upgrade: UpgradeId, queue: bool = False, can_afford_check: bool = False
+    ) -> Union[UnitCommand, bool]:
         """ Orders unit to research 'upgrade'.
         Requires UpgradeId to be passed instead of AbilityId.
 
@@ -1270,7 +1307,9 @@ class Unit:
             can_afford_check=can_afford_check,
         )
 
-    def warp_in(self, unit: UnitTypeId, position: Union[Point2, Point3], can_afford_check: bool = False) -> Union[UnitCommand, bool]:
+    def warp_in(
+        self, unit: UnitTypeId, position: Union[Point2, Point3], can_afford_check: bool = False
+    ) -> Union[UnitCommand, bool]:
         """ Orders Warpgate to warp in 'unit' at 'position'.
 
         :param unit:
